--{{{  MSG PROTOCOL
--{{{  HISTORY
-- HISTORY:
-- 08.12.92 Destination is included in echo.read.abs tag of the protocol
-- because it's necessary to know where it comes from.

-- 08.12.92 Tags read.loc, read.abs, echo.read.abs, echo.read.loc are eliminated
-- remaining tags: read, echo.read
--}}}  

PROTOCOL MSG
  CASE
    --{{{  
    --{{{  COMMENTED OLD ACTIVATION
    -- activation; INT; INT; INT :: []INT; INT :: []INT
					-- Signal to activate parallel mode
					-- 1: Destination Transputer
					-- 2: PC where execution must begin
					-- 3: Number of processors becoming active
					-- 4: Base registers
					-- 5: Number of processors becoming active
					-- 6: Name registers
    
    --}}}  
    activation; INT; INT; INT :: []INT
					-- Signal to activate parallel mode
					-- 1: PC where execution must begin
					-- 2: Number of processors becoming active
					-- 3: Array(4) number of elements.
					-- 4: Quantity + their Base & name registers
    
    relax.activation; INT; INT; INT :: []INT
					-- Signal to activate parallel mode
					-- 1: PC where execution must begin
					-- 2: Number of processors becoming active
					-- 3: Array(4) number of elements.
					-- 4: Quantity + their Base & name registers
    
    e.o.program; INT                    -- Indicates end of program. 1: Transputer
    echo.readint2; INT                  -- Result of a keyboard read
					-- 1: Read value
    echo.readln                         -- Notification of a successful keyboard readln
    prog.sent; BOOL; INT; INT :: []INT  -- Program Array.
					-- 1: Indicates debug mode 2: Initial PC for debug 3: Length 4: Array
    syncro.end.pattern;  INT            -- 1: Local PC
    syncro.end.nested.pattern;  INT     -- 1: Local PC
    syncro.root                         -- Sinchronization signal from root to slave
    syncro.root.assign
    syncro.loop; BOOL; INT              -- 1: Active 2: Local PC
    syncro.slave; BOOL; INT             -- 1: Active 2: Local PC
    syncro.wake.up;  INT                -- 1: Local PC
    syncro.assign                       --
    
    write; INT; INT                     -- Write Memory
					-- 1: Address
					-- 2: Value to be written
    replicate.write; INT; INT           -- Memory replication
					-- 1: Address
					-- 2: Value to be written
    replicate.mass; INT :: []INT        -- Massive Memory replication
					-- 1: Length
					-- 2: Array of Addresses-Values
    replicate.mass.from; INT; INT :: []INT
					-- Massive Memory replication from address
					-- 1: Initial address
					-- 2: Length
					-- 3: Array of Values
    --{{{  to.mon.codes
    ----------------------------------------------------------------------------------------
    to.mon.e.o.program; INT             -- Indicates end of program
					-- 1: Transputer
    to.mon.instr.debug; INT; INT; INT; BOOL
					-- Instructions Debug
					-- 1: Transputer
					-- 2: PC
					-- 3: Instrucction number
					-- 4: <<Press Key>> is required
    to.mon.prog.read; INT               -- Program reception 1: Transputer
    to.mon.readint2                     -- Read integer value from keyboard.
    to.mon.readln                       -- Read a carriage return from keyboard.
    to.mon.writeint2; INT               -- Write integer value to screen.
					-- 1: Integer Value.
    to.mon.writestr2; INT :: []BYTE     -- Write String
					-- 1: Length 2: String
    to.mon.wstr.int; INT :: []BYTE; INT -- Write String & Integer
					-- 1: Length 2: String 3: Integer
    -- to.mon.elapsed.time; INT; INT       -- Program execution time.
    --                                     -- 1: Ticks time 2: Time in seconds
    to.mon.string; INT :: []BYTE; BOOL  -- Write String (usually contains text and integers)
					-- 1: Length
					-- 2: String
					-- 3: <<Press Key>> is required
    ----------------------------------------------------------------------------------------
    to.mon.err; INT; INT; INT           -- 1: Error code 2: Transputer 3: Additional value
    --}}}  
    --}}}  
:
--}}}  

--{{{  Instructions, instruction names, instruction offsets & synchronizations

--{{{  INSTRUCTIONS
--{{{  DOCUMENTATION (Pseudoinstructions)
-- defaddr2, defarg2 and pushconst are Pseudoinstruction. They are never
-- generated by the compiler. They would be used when programming directly
-- in PPRAM assembler.
--}}}  

VAL INT active2                IS 0   :
VAL INT add2                   IS 1   :
VAL INT and2                   IS 2   :
VAL INT and.int2               IS 3   : -- (new1: 10.01.94)
VAL INT assign2                IS 4   :
VAL INT broadcast2             IS 5   : -- (new3: 09.05.94)
VAL INT complement2            IS 6   :
VAL INT constant2              IS 7   :
VAL INT copy.no.tops2          IS 8   :
VAL INT copy.tops2             IS 9   :
VAL INT debug2                 IS 10  : -- (new2: 11.04.94)
VAL INT def.addr2              IS 11  :
VAL INT def.arg2               IS 12  :
VAL INT divide2                IS 13  :
VAL INT do2                    IS 14  :
VAL INT end.block2             IS 15  :
VAL INT end.nested.pattern2    IS 16  :
VAL INT end.pattern2           IS 17  :
VAL INT end.proc2              IS 18  :
VAL INT end.prog2              IS 19  :
VAL INT equal2                 IS 20  :
VAL INT equal.int2             IS 21  : -- (new1: 10.01.94)
VAL INT end.relax2             IS 22  :
VAL INT end.single.pattern2    IS 23  :
VAL INT field2                 IS 24  :
VAL INT fill2                  IS 25  :
VAL INT fill.zero2             IS 26  :
VAL INT free2                  IS 27  :
VAL INT get.time2              IS 28  :
VAL INT global.call1           IS 29  : -- (new1: 10.01.94)
VAL INT global.call2           IS 30  : -- (new1: 10.01.94)
VAL INT global.call3           IS 31  : -- (new1: 10.01.94)
VAL INT global.value1          IS 32  : -- (new1: 10.01.94)
VAL INT global.value2          IS 33  : -- (new1: 10.01.94)
VAL INT global.value3          IS 34  : -- (new1: 10.01.94)
VAL INT global.var1            IS 35  : -- (new1: 10.01.94)
VAL INT global.var2            IS 36  : -- (new1: 10.01.94)
VAL INT global.var3            IS 37  : -- (new1: 10.01.94)
VAL INT go.if.no.zero2         IS 38  :
VAL INT go.if.zero2            IS 39  :
VAL INT goto2                  IS 40  :
VAL INT greater2               IS 41  :
VAL INT greater.int2           IS 42  : -- (new1: 10.01.94)
VAL INT in2                    IS 43  :
VAL INT index2                 IS 44  :
VAL INT init.for2              IS 45  : -- (new2: 11.04.94)
VAL INT init.nested.for2       IS 46  : -- (new2: 11.04.94)
VAL INT less2                  IS 47  :
VAL INT less.int2              IS 48  : -- (new1: 10.01.94)
VAL INT local.call2            IS 49  : -- (new1: 10.01.94)
VAL INT local.value2           IS 50  :
VAL INT local.var2             IS 51  :
VAL INT loop2                  IS 52  : -- (new2: 11.04.94)
VAL INT marked2                IS 53  :
VAL INT max2                   IS 54  :
VAL INT min2                   IS 55  :
VAL INT minus2                 IS 56  :
VAL INT modulo2                IS 57  :
VAL INT multiply2              IS 58  :
VAL INT nested.loop2           IS 59  : -- (new2: 11.04.94)
VAL INT nested.pattern2        IS 60  : -- (modified: 10.01.94)
VAL INT new.block2             IS 61  :
VAL INT new.line2              IS 62  :
VAL INT not2                   IS 63  :
VAL INT not.int2               IS 64  : -- (new1: 10.01.94)
VAL INT not.do2                IS 65  :
VAL INT not.equal2             IS 66  :
VAL INT not.equal.int2         IS 67  : -- (new1: 10.01.94)
VAL INT not.greater2           IS 68  :
VAL INT not.greater.int2       IS 69  : -- (new1: 10.01.94)
VAL INT not.less2              IS 70  :
VAL INT not.less.int2          IS 71  : -- (new1: 10.01.94)
VAL INT or2                    IS 72  :
VAL INT or.int2                IS 73  : -- (new1: 10.01.94)
VAL INT pattern2               IS 74  : -- (modified: 10.01.94)
VAL INT pop.pattern2           IS 75  :
VAL INT private.assign2        IS 76  :
VAL INT private.get.time2      IS 77  :
VAL INT private.global.call1   IS 78  : -- (new1: 10.01.94)
VAL INT private.global.call2   IS 79  : -- (new1: 10.01.94)
VAL INT private.global.call3   IS 80  : -- (new1: 10.01.94)
VAL INT private.local.call2    IS 81  : -- (new1: 10.01.94)
VAL INT private.new.block2     IS 82  : -- (new1: 10.01.94)
VAL INT private.proc.call2     IS 83  : -- (new1: 10.01.94)
VAL INT private.read.int2      IS 84  :
VAL INT private.seg.assign2    IS 85  : -- (new3: 09.05.94)
VAL INT private.set.time2      IS 86  :
VAL INT private.simple.assign2 IS 87  :
VAL INT proc.call2             IS 88  :
VAL INT procedure2             IS 89  :
VAL INT program2               IS 90  :
VAL INT push.const2            IS 91  :
VAL INT push.counter2          IS 92  : -- (new2: 11.04.94)
VAL INT push.name2             IS 93  :
VAL INT push.pattern2          IS 94  :
VAL INT random2                IS 95  :
VAL INT randomize2             IS 96  :
VAL INT read.int2              IS 97  :
VAL INT readln2                IS 98  :
VAL INT relax2                 IS 99  :
VAL INT seg.assign2            IS 100 : -- (new3: 09.05.94)
VAL INT seg.broadcast2         IS 101 : -- (new3: 09.05.94)
VAL INT seg.dyn.init2          IS 102 : -- (new3: 09.05.94)
VAL INT seg.index2             IS 103 : -- (new3: 09.05.94)
VAL INT seg.init2              IS 104 : -- (new3: 09.05.94)
VAL INT set.time2              IS 105 :
VAL INT share2                 IS 106 :
VAL INT shl2                   IS 107 :
VAL INT shl.int2               IS 108 : -- (new1: 10.01.94)
VAL INT shr2                   IS 109 :
VAL INT shr.int2               IS 110 : -- (new1: 10.01.94)
VAL INT simple.assign2         IS 111 :
VAL INT simple.value2          IS 112 :
VAL INT single.control2        IS 113 :
VAL INT single.pattern2        IS 114 :
VAL INT size2                  IS 115 : -- (new3: 09.05.94)
VAL INT subtract2              IS 116 :
VAL INT value2                 IS 117 :
VAL INT variable2              IS 118 :
VAL INT var.param2             IS 119 :
VAL INT write.int2             IS 120 :
VAL INT write.str2             IS 121 :
VAL INT xor2                   IS 122 :
VAL INT xor.int2               IS 123 :
--}}}                           
--{{{  offsets of the instructions
VAL []INT offset IS
[1, -- active
1, -- add
2, -- and                              
1, -- and.int
2, -- assign                           
2, -- broadcast2             
1, -- complement                       
2, -- constant                         
1, -- copy.no.tops
1, -- copy.tops
1, -- debug
1, -- def.addr
1, -- def.arg
1, -- divide                           
2, -- do                               
1, -- end.block
1, -- end.nested.pattern
1, -- end.pattern
2, -- end.proc
1, -- end.prog
2, -- equal                            
1, -- equal.int
1, -- end.relax
1, -- end.single.pattern
2, -- field                            
5, -- fill                             
2, -- fill.zero
2, -- free                             
1, -- get.time
2, -- global.call1
2, -- global.call
2, -- global.call3
2, -- global.value1
2, -- global.value
2, -- global.value3
2, -- global.var1
2, -- global.var
2, -- global.var3
2, -- go.if.no.zero
2, -- go.if.zero
2, -- goto                             
2, -- greater                          
1, -- greater.int
2, -- in.                             
5, -- index                            
2, -- init.for
2, -- init.nested.for
2, -- less                             
1, -- less.int
2, -- local.call
2, -- local.value
2, -- local.var
3, -- loop
5, -- marked                           
1, -- max                              
1, -- min                              
1, -- minus                            
1, -- modulo                           
1, -- multiply
3, -- nested.loop
3, -- nested.pattern
2, -- new.block
2, -- new.line
2, -- not                              
1, -- not.int
2, -- not.do
2, -- not.equal
1, -- not.equal.int
2, -- not.greater
1, -- not.greater.int
2, -- not.less
1, -- not.less.int
2, -- or                               
1, -- or.int
3, -- pattern                          
1, -- pop.pattern
2, -- private.assign
1, -- private.get.time
2, -- private.global.call1
2, -- private.global.call
2, -- private.global.call3
2, -- private.local.call
2, -- private.new.block
3, -- private.proc.call
1, -- private.read.int
3, -- private.seg.assign2    
1, -- private.set.time
1, -- private.simple.assign
3, -- proc.call
5, -- procedure                        
5, -- program                          
2, -- push.const
1, -- push.counter
2, -- push.name
1, -- push.pattern
2, -- random                           
1, -- randomize                        
1, -- read.int
1, -- readln.int
1, -- relax                            
3, -- seg.assign2            
2, -- seg.broadcast2         
3, -- seg.dyn.init2          
3, -- seg.index2             
5, -- seg.init2              
1, -- set.time
2, -- share                            
2, -- shl                              
1, -- shl.int
2, -- shr                              
1, -- shr.int
1, -- simple.assign
1, -- simple.value
2, -- single.control
2, -- single.pattern
1, -- size2                  
1, -- subtract                         
2, -- value                            
3, -- variable                         
3, -- var.param
1, -- write.int
3, -- writestr                         
2, -- xor                              
1] : -- xorint                         
--}}}  
VAL INT no.of.instructions IS SIZE offset : -- Number of instructions
--{{{  instruction names
VAL [no.of.instructions][]BYTE instr.name IS
["ACTIVE              ",  --  active                   
"ADD                 ",  --  add                       
"AND                 ",  --  and
"ANDINT              ",  --  andint
"ASSIGN              ",  --  assign                    
"BROADCAST           ",  -- broadcast              
"COMPLEMENT          ",  --  complement                
"CONSTANT            ",  --  constant                  
"COPYNOTOPS          ",  --  copynotops                
"COPYTOPS            ",  --  copytops                  
"DEBUG               ",  --  debug
"DEFADDR             ",  --  defaddr                   
"DEFARG              ",  --  defarg                    
"DIVIDE              ",  --  divide
"DO                  ",  --  do                        
"ENDBLOCK            ",  --  endblock                  
"ENDNESTEDPATTERN    ",  -- endnestedpattern           
"ENDPATTERN          ",  -- endpattern                  
"ENDPROC             ",  --  endproc                   
"ENDPROG             ",  --  endprog                   
"EQUAL               ",  --  equal                     
"EQUALINT            ",  --  equalint                  
"ENDRELAX            ",  --  endrelax                  
"ENDSINGLEPATTERN    ",  -- endsinglepattern           
"FIELD               ",  --  field                     
"FILL                ",  --  fill                      
"FILLZERO            ",  --  fillzero                  
"FREE                ",  --  free                      
"GETTIME             ",  --  gettime                   
"GLOBALCALL12,       ",  --  globalcall1               
"GLOBALCALL22,       ",  --  globalcall               
"GLOBALCALL32,       ",  --  globalcall3               
"GLOBALVALUE12,      ",  --  globalvalue1              
"GLOBALVALUE22,      ",  --  globalvalue              
"GLOBALVALUE32,      ",  --  globalvalue3              
"GLOBALVAR12,        ",  --  globalvar1                
"GLOBALVAR22,        ",  --  globalvar                
"GLOBALVAR32,        ",  --  globalvar3                
"GOIFNOZERO          ",  --  goifnozero                
"GOIFZERO            ",  --  goifzero                  
"GOTO                ",  --  goto                      
"GREATER             ",  --  greater                   
"GREATERINT          ",  --  greaterint                
"IN                  ",  --  in.                      
"INDEX               ",  --  index                     
"INIT.FOR            ",  --  initfor
"INIT.NESTED.FOR     ",  --  init.nested.for
"LESS                ",  --  less                      
"LESSINT             ",  --  lessint                   
"LOCALCALL           ",  --  localcall                 
"LOCALVALUE          ",  --  localvalue
"LOCALVAR            ",  --  localvar                  
"LOOP                ",  --  loop
"MARKED              ",  --  marked                    
"MAX                 ",  --  max                       
"MIN                 ",  --  min                       
"MINUS               ",  --  minus                     
"MODULO              ",  --  modulo                    
"MULTIPLY            ",  --  multiply
"NESTED.LOOP         ",  --  nested.loop
"NESTEDPATTERN       ",  --  nestedpattern             
"NEWBLOCK            ",  --  newblock                  
"NEWLINE             ",  --  newline                   
"NOT                 ",  --  not                       
"NOTINT              ",  --  notint                    
"NOTDO               ",  --  notdo
"NOTEQUAL            ",  --  notequal                  
"NOTEQUALINT         ",  --  notequalint               
"NOTGREATER          ",  --  notgreater                
"NOTGREATERINT       ",  --  notgreaterint             
"NOTLESS             ",  --  notless                   
"NOTLESSINT          ",  --  notlessint                
"OR                  ",  --  or                        
"ORINT               ",  --  orint                     
"PATTERN             ",  --  pattern                   
"POPPATTERN          ",  --  poppattern                
"PRIVATEASSIGN       ",  --  private.assign
"PRIVATEGETTIME      ",  --  private.gettime
"PRIVATEGLOBALCALL12 ",  --  private.globalcall1
"PRIVATEGLOBALCALL22 ",  --  private.globalcall
"PRIVATEGLOBALCALL32 ",  --  private.globalcall3
"PRIVATELOCALCALL2,  ",  --  private.localcall,
"PRIVATENEWBLOCK2,   ",  --  private.newblock,
"PRIVATEPROCCALL2    ",  --  private.proccall,
"PRIVATEREADINT      ",  --  private.readint
"PRIVATE.SEG.ASSIGN  ",  --  private.seg.assign     
"PRIVATESETTIME      ",  --  private.settime
"PRIVATESIMPLEASSIGN ",  --  private.simpleassign
"PROCCALL            ",  --  proccall                  
"PROCEDURE           ",  --  procedure                 
"PROGRAM             ",  --  program                   
"PUSHCONST           ",  --  pushconst                 
"PUSHCOUNTER         ",  --  pushcounter
"PUSHNAME            ",  --  pushname                  
"PUSHPATTERN         ",  --  pushpattern               
"RANDOM              ",  --  random                     
"RANDOMIZE           ",  --  randomize                  
"READINT             ",  --  readint                   
"READLN              ",  --  readln                    
"RELAX               ",  --  relax                     
"SEG.ASSIGN          ",  --  seg.assign             
"SEG.BROADCAST       ",  --  seg.broadcast          
"SEG.DYN.INIT        ",  --  seg.dyn.init           
"SEG.INDEX           ",  --  seg.index              
"SEG.INIT            ",  --  seg.init               
"SETTIME             ",  --  settime                   
"SHARE               ",  --  share                     
"SHL                 ",  --  shl                       
"SHLINT              ",  --  shlint                    
"SHR                 ",  --  shr                       
"SHRINT              ",  --  shrint                    
"SIMPLEASSIGN        ",  --  simpleassign              
"SIMPLEVALUE         ",  --  simplevalue               
"SINGLECONTROL       ",  --  singlecontrol             
"SINGLEPATTERN       ",  --  singlepattern             
"SIZE                ",  --  size                   
"SUBTRACT            ",  --  subtract                  
"VALUE               ",  --  value                     
"VARIABLE            ",  --  variable                  
"VARPARAM            ",  --  varparam                  
"WRITEINT            ",  --  writeint                  
"WRITESTR            ",  --  writestr                  
"XOR                 ",  --  xor                       
"XORINT              "] :  --  xorint                  
--}}}  
--}}}  
--{{{  General constants
VAL INT word.pram           IS 32 :
VAL INT word.pram.minus.1   IS 31 :
VAL INT log2.word.pram      IS 5 :
VAL INT global.ctrl.name    IS -1 :  -- Arbitrary index for global control processor
VAL INT max.prog.size       IS 10000 : -- Maximum Program size
VAL INT min.address         IS 0 :
VAL INT rubbish             IS -9000009 : -- Rubbish value (A non-important one)
VAL INT max.width           IS 3 :

VAL INT size.of.activation.record IS 3 :
--{{{  COMMENT to size.of.activation.record
-- When the activation signal is sent to a transputer, with the signal we
-- send the activation context: the values for the base and name registers of
-- the PRAM processors that become active. This context is sent in an array
-- where the components are considered in groups of 3:

-- ac[3 * i]: A certain number of processor
-- ac[3 * i + 1] The value of the Base Register for these (ac[3*i]) processors
-- ac[3 * i + 2] The value of the Name Register for the first of these processors

-- size.of.activation.record is the size of this record. This constant is used
-- instead of the value 3.
--}}}  

VAL INT max.packet.size     IS 512 : -- It must be even!!!
				     -- The number of individual messages will
				     -- be max.packet.size / 2 = 256

VAL INT max.message         IS 2048 : -- See the comment downline
--{{{  COMMENT to max.message
-- VAL INT max.message      IS (2 * max.procs.per.trans) : -- See the comment downline



-- Maximum size of any message.

-- Must be bigger than max.packet.size, because packet.size is used as an
-- index for the array of messages.

-- The bigest message arises when a write.mass operation is done.
-- When the message sent is the asociated with the write.mass operation, the
-- size of the message is (2 * num.active) and the maximum of this quantity
-- is (2 * max.procs.per.trans)
--
-- When the activation context is sent, the array needed can be of size
-- max.procs.per.trans at most because the context is sent when it is
-- built, this is, in a pipeline fashion.
--}}}  
--}}}  
